searchNodes=[{"ref":"Mimic.html","title":"Mimic","type":"module","doc":"Mimic is a library that simplifies the usage of mocks in Elixir. Mimic is mostly API compatible with mox but doesn&#39;t require explicit contract checking with behaviours. It&#39;s also faster. You&#39;re welcome. Mimic works by copying your module out of the way and replacing it with one of it&#39;s own which can delegate calls back to the original or to a mock function as required. In order to prepare a module for mocking you must call copy/1 with the module as an argument. We suggest that you do this in your test/test_helper.exs: Mimic.copy(Calculator) ExUnit.start() Importantly calling copy/1 will not change the behaviour of the module. When writing tests you can then use stub/3 or expect/3 to add mocks and assertions. Multi-process collaboration Mimic supports multi-process collaboration via two mechanisms: Explicit allows. Global mode. Using explicit allows is generally preferred as these stubs can be run concurrently, whereas global mode tests must be run exclusively. Explicit allows Using allow/3 you can give other processes permission to use stubs and expectations from where they were not defined. test &quot;invokes add from a task&quot; do Caculator |&gt; expect(:add, fn x, y -&gt; x + y end) parent_pid = self() Task.async(fn -&gt; Calculator |&gt; allow(parent_pid, self()) assert Calculator.add(2, 3) == 5 end) |&gt; Task.await end Global mode When set in global mode any process is able to call the stubs and expectations defined in your tests. Warning: If using global mode you should remove async: true from your tests Enable global mode using set_mimic_global/1. setup :set_mimic_global setup :verify_on_exit! test &quot;invokes add from a task&quot; do Calculator |&gt; expect(:add, fn x, y -&gt; x + y end) Task.async(fn -&gt; assert Calculator.add(2, 3) == 5 end) |&gt; Task.await end"},{"ref":"Mimic.html#allow/3","title":"Mimic.allow/3","type":"function","doc":"Allow other processes to share expectations and stubs defined by another process. Arguments: module - the copied module. owner_pid - the process ID of the process which created the stub. allowed_pid - the process ID of the process which should also be allowed to use this stub. Raises: If Mimic is running in global mode. Allows other processes to share expectations and stubs defined by another process. Example test &quot;invokes add from a task&quot; do Caculator |&gt; expect(:add, fn x, y -&gt; x + y end) parent_pid = self() Task.async(fn -&gt; Calculator |&gt; allow(parent_pid, self()) assert Calculator.add(2, 3) == 5 end) |&gt; Task.await end"},{"ref":"Mimic.html#copy/1","title":"Mimic.copy/1","type":"function","doc":"Prepare module for mocking. Arguments: module - the name of the module to copy."},{"ref":"Mimic.html#expect/4","title":"Mimic.expect/4","type":"function","doc":"Define a stub which must be called within an example. This function is almost identical to stub/3 except that the replacement function must be called within the lifetime of the calling pid (i.e. the test example). Arguments: module - the name of the module in which we&#39;re adding the stub. function_name - the name of the function we&#39;re stubbing. function - the function to use as a replacement. Raises: If module is not copied. If function_name is not publicly exported from module with the same arity. If function is not called by the stubbing process. Example iex&gt; Calculator.add(2, 4) 6 iex&gt; Mimic.expect(Calculator, :add, fn x, y -&gt; x * y end) ...&gt; Calculator.add(2, 4) 8"},{"ref":"Mimic.html#reject/1","title":"Mimic.reject/1","type":"function","doc":"Define a stub which must not be called. This function allows you do define a stub which must not be called during the course of this test. If it is called then the verification step will raise. Arguments: function - A capture of the function which must not be called. Raises: If function is not called by the stubbing process while calling verify!/1. Example: iex&gt; Mimic.reject(&amp;Calculator.add/2) Calculator"},{"ref":"Mimic.html#reject/3","title":"Mimic.reject/3","type":"function","doc":"Define a stub which must not be called. This function allows you do define a stub which must not be called during the course of this test. If it is called then the verification step will raise. Arguments: module - the name of the module in which we&#39;re adding the stub. function_name - the name of the function we&#39;re stubbing. arity - the arity of the function we&#39;re stubbing. Raises: If function is not called by the stubbing process while calling verify!/1. Example: iex&gt; Mimic.reject(Calculator, :add, 2) Calculator"},{"ref":"Mimic.html#set_mimic_from_context/1","title":"Mimic.set_mimic_from_context/1","type":"function","doc":"Chooses the mode based on ExUnit context. If async is true then the mode is private, otherwise global. setup :set_mimic_from_context"},{"ref":"Mimic.html#set_mimic_global/1","title":"Mimic.set_mimic_global/1","type":"function","doc":"Sets the mode to global. Mocks can be set and used by all processes setup :set_mimic_global"},{"ref":"Mimic.html#set_mimic_private/1","title":"Mimic.set_mimic_private/1","type":"function","doc":"Sets the mode to private. Mocks can be set and used by the process setup :set_mimic_private"},{"ref":"Mimic.html#stub/1","title":"Mimic.stub/1","type":"function","doc":"Replace all public functions in module with stubs. The stubbed functions will raise if they are called. Arguments: module - The name of the module to stub. Raises: If module is not copied. If function is not called by the stubbing process. Example iex&gt; Mimic.stub(Calculator) ...&gt; Calculator.add(2, 4) ** (ArgumentError) Module Calculator has not been copied. See docs for Mimic.copy/1"},{"ref":"Mimic.html#stub/3","title":"Mimic.stub/3","type":"function","doc":"Define a stub function for a copied module. Arguments: module - the name of the module in which we&#39;re adding the stub. function_name - the name of the function we&#39;re stubbing. function - the function to use as a replacement. Raises: If module is not copied. If function_name is not publicly exported from module with the same arity. Example iex&gt; Calculator.add(2, 4) 6 iex&gt; Mimic.stub(Calculator, :add, fn x, y -&gt; x * y end) ...&gt; Calculator.add(2, 4) 8"},{"ref":"Mimic.html#verify!/1","title":"Mimic.verify!/1","type":"function","doc":"Verify if expectations were fulfilled for a process pid"},{"ref":"Mimic.html#verify_on_exit!/1","title":"Mimic.verify_on_exit!/1","type":"function","doc":"Verifies the current process after it exits. If you want to verify expectations for all tests, you can use verify_on_exit!/1 as a setup callback: setup :verify_on_exit!"},{"ref":"Mimic.Cover.html","title":"Mimic.Cover","type":"module","doc":"Abuse cover private functions to move stuff around Completely based on meck&#39;s solution: https://github.com/eproxus/meck/blob/2c7ba603416e95401500d7e116c5a829cb558665/src/meck_cover.erl#L67-L91"},{"ref":"readme.html","title":"Mimic","type":"extras","doc":"Mimic A sane way of using mocks in Elixir. It borrows a lot from both Meck &amp; Mox! Thanks @eproxus &amp; @josevalim"},{"ref":"readme.html#installation","title":"Mimic - Installation","type":"extras","doc":"Just add mimic to your list of dependencies in mix.exs: def deps do [ {:mimic, &quot;~&gt; 1.0&quot;, only: :test} ] end If :applications key is defined inside your mix.exs or you run mix test --no-start, you probably want to add Application.ensure_all_started(:mimic) in your test_helper.exs"},{"ref":"readme.html#using","title":"Mimic - Using","type":"extras","doc":"Modules need to be prepared so that they can be used. You must first call copy in your test_helper.exs for each module that may have the behaviour changed. Mimic.copy(Calculator) ExUnit.start() Calling copy will not change the behaviour of the module. The user must call stub/1, stub/3, expect/4 or reject/1 so that the functions can behave differently. Then for the actual tests one could use it like this: use ExUnit.Case, async: true use Mimic test &quot;invokes add once and mult twice&quot; do Calculator |&gt; stub(:add, fn x, y -&gt; :stub end) |&gt; expect(:add, fn x, y -&gt; x + y end) |&gt; expect(:mult, 2, fn x, y -&gt; x * y end) assert Calculator.add(2, 3) == 5 assert Calculator.mult(2, 3) == 6 assert Calculator.add(2, 3) == :stub end"},{"ref":"readme.html#stub-expect-and-reject","title":"Mimic - Stub, Expect and Reject","type":"extras","doc":"Stub stub/1 will change every module function to throw an exception if called. stub(Calculator) ** (Mimic.UnexpectedCallError) Stub! Unexpected call to Calculator.add(3, 7) from #PID&lt;0.187.0&gt; code: assert Calculator.add(3, 7) == 10 stub/3 changes a specific function to behave differently. If the function is not called no verification error will happen. Expect expect/4 changes a specific function and it works like a queue of operations. It has precedence over stubs and if not called a verification error will be thrown. If the same function is called with expect/4 the order will be respected: Calculator |&gt; stub(:add, fn _x, _y -&gt; :stub end) |&gt; expect(:add, fn _, _ -&gt; :expected_1 end) |&gt; expect(:add, fn _, _ -&gt; :expected_2 end) assert Calculator.add(1, 1) == :expected_1 assert Calculator.add(1, 1) == :expected_2 assert Calculator.add(1, 1) == :stub expect/4 has an optional parameter which is the amount of calls expected: Calculator |&gt; expect(:add, 2, fn x, y -&gt; {:add, x, y} end) assert Calculator.add(1, 3) == {:add, 1, 3} assert Calculator.add(4, 5) == {:add, 4, 5} Reject One may want to reject calls to a specific function. reject/1 can be used to achieved this behaviour. reject(&amp;Calculator.add/2) assert_raise Mimic.UnexpectedCallError, fn -&gt; Calculator.add(4, 2) end"},{"ref":"readme.html#private-and-global-mode","title":"Mimic - Private and Global mode","type":"extras","doc":"The default mode is private which means that only the process and explicitly allowed process will see the different behaviour. Calling allow/2 will permit a different pid to call the stubs and expects from the original process. Global mode can be used with set_mimic_global like this: setup :set_mimic_global test &quot;invokes add and mult&quot; do Calculator |&gt; expect(:add, fn x, y -&gt; x + y end) |&gt; expect(:mult, fn x, y -&gt; x * y end) Task.async(fn -&gt; assert Calculator.add(2, 3) == 5 assert Calculator.mult(2, 3) == 6 end) |&gt; Task.await end This means that all processes will get the same behaviour defined with expect &amp; stub. This option is simpler but tests running concurrently will have undefined behaviour. It is important to run with async: false. One could use :set_mimic_from_context instead of using :set_mimic_global or :set_mimic_private. It will be private if async: true, global otherwise."},{"ref":"readme.html#implementation-details-performance","title":"Mimic - Implementation Details &amp; Performance","type":"extras","doc":"After calling Mimic.copy(MyModule), calls to functions belonging to this module will first go through an ETS table to check which pid sees what (stubs, expects or call original). It is really fast but it won&#39;t be as fast as calling a no-op function. Here&#39;s a very simple benchmark: defmodule Enumerator do def to_list(x, y), do: Enum.to_list(x..y) end Benchmarking Enumerator.to_list(1, 100) : Name ips average deviation median 99th % mimic 116.00 K 8.62 μs ±729.13% 5 μs 29 μs original 19.55 K 51.15 μs ±302.46% 34 μs 264 μs Comparison: mimic 116.00 K original 19.55 K - 5.93x slower Benchmarking Enumerator.to_list(1, 250) : Name ips average deviation median 99th % original 131.49 K 7.61 μs ±167.90% 7 μs 16 μs mimic 105.47 K 9.48 μs ±145.21% 9 μs 27 μs Comparison: original 131.49 K mimic 105.47 K - 1.25x slower There&#39;s a small fixed price to pay when mimic is used but it is unnoticeable for tests purposes."},{"ref":"readme.html#acknowledgements","title":"Mimic - Acknowledgements","type":"extras","doc":"Thanks to @jamesotron and @alissonsales for all the help! :tada:"}]